(function ($, Drupal, drupalSettings) {
  Drupal.behaviors.block_ajax = {
    attach: function (context, drupalSettings) {
      /**
       * Ajax Block Handler.
       *
       * @param $block
       */
      let ajaxBlockHandler = function ($block) {
        let blockId = $block.data('block-ajax-id');
        if (blockId && drupalSettings.block_ajax.blocks[blockId] !== undefined) {
          let pluginId = $block.data('block-ajax-plugin-id');
          if (!pluginId) {
            return;
          }
          let request = $.ajax({
            type: "POST",
            url: Drupal.url('block/ajax/' + blockId),
            data: {
              plugin_id: pluginId,
              config: drupalSettings.block_ajax.blocks[blockId]
            },
            timeout: 0,
            cache: false,
            async: true,
            dataType: 'json'
          });
          request.done(function(data) {
            if (typeof data.content !== 'string' || !data.content.length) {
              return;
            }
            // Add processed class to block wrapper.
            $block.parent().addClass('block-ajax-processed');
            // Compose a new block content, replacing the old one
            // with it.
            let $newBlock = $(data.content);
            $block.replaceWith($newBlock);
            // Attach behaviours to the updated element.
            $newBlock.each(function () {
              // Note that the global drupal settings reference is being used to
              // make sure the actualised global state is being used.
              Drupal.attachBehaviors(this, window.drupalSettings);
            });
          });
          request.fail(function(jqXHR, textStatus) {
            console.log("Ajax Block: " + blockId + " request failed: " + textStatus);
          });
        }
      };
      /**
       * Initialize and loop over Ajax blocks.
       */
      $('[data-block-ajax-id]', context).once('block_ajax').each(function () {
        let $block = $(this);
        // Load in block via AJAX
        ajaxBlockHandler($block);
        // On RefreshAjaxBlock event
        $block.on('RefreshAjaxBlock', function () {
          // Execute the handler payload.
          ajaxBlockHandler($(this));
        });
      });
    },
    detach: function (context) {
    }
  };
  /**
   * Implements ajax block refreshing command.
   */
  Drupal.AjaxCommands.prototype.AjaxBlockRefreshCommand = function (ajax, response, status) {
    $(response.selector).trigger('RefreshAjaxBlock');
  };
})(jQuery, Drupal, drupalSettings);
