<?php

namespace Drupal\block_ajax;

use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Cache\CacheTagsInvalidator;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Ajax Blocks Service.
 */
class AjaxBlocks {

  use StringTranslationTrait;

  /**
   * The config factory object.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The cache tags invalidator.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidator
   */
  protected $cacheTagsInvalidator;

  /**
   * Ajax Blocks Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory object.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user object.
   */
  public function __construct(ConfigFactory $config_factory, AccountProxyInterface $current_user, CacheTagsInvalidator $cache_tags_invalidator) {
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->cacheTagsInvalidator = $cache_tags_invalidator;
  }

  /**
   * Check to see if user has proper permissions/access rights.
   *
   * @return bool
   *   Returns access right.
   */
  public function hasAccess(): bool {
    return $this->currentUser->hasPermission('administer ajax blocks');
  }

  /**
   * Check if given block is configured as an AJAX block.
   *
   * @param \Drupal\Core\Block\BlockPluginInterface $block
   *   Block plugin.
   *
   * @return bool
   *   Returns TRUE if current block is AJAX block.
   */
  public function isAjaxBlock(BlockPluginInterface $block): bool {
    $conf = $block->getConfiguration();
    return !empty($conf['block_ajax']['is_ajax']);
  }

  /**
   * Invalidate ajax block cache tags.
   *
   * @return void
   */
  public function invalidateAjaxBlocks() {
    $this->cacheTagsInvalidator->invalidateTags(['block_ajax']);
  }
}
