<?php

namespace Drupal\block_ajax\Controller;

use Drupal\Core\Block\BlockManager;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Ajax Blocks Controller.
 */
class AjaxBlockController extends ControllerBase {

  /**
   * @var \Drupal\Core\Block\BlockManager
   */
  protected $blockManager;

  /**
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Block Ajax Controller Constructor.
   *
   * @param \Drupal\Core\Block\BlockManager $block_manager
   * @param \Drupal\Core\Render\RendererInterface $renderer
   */
  public function __construct(BlockManager $block_manager, RendererInterface $renderer) {
    $this->blockManager = $block_manager;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.block'),
      $container->get('renderer')
    );
  }

  /**
   * Implements ajax block update request handler.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   * @param string $block_id
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Exception
   */
  public function loadBlock(Request $request, string $block_id = ''): JsonResponse {
    // Check for block id.
    if (empty($block_id)) {
      return new JsonResponse();
    }

    // Make sure we have plugin id.
    $plugin_id = $request->get('plugin_id', '');
    if (empty($plugin_id)) {
      return new JsonResponse();
    }

    // Grab any configuration passed in.
    $configuration = $request->get('config', []);

    // Construct and render the block.
    $block = $this->blockManager->createInstance($plugin_id, $configuration)
      ->build();
    $block = $this->renderer->renderRoot($block);

    return new JsonResponse([
      'block_id' => $block_id,
      'plugin_id' => $plugin_id,
      'content' => trim($block),
    ]);
  }

}
